@@ -2,6 +2,7 @@ from BeautifulSoup import BeautifulSoup
 from ftw.pdfgenerator import interfaces
 from ftw.pdfgenerator.html2latex import subconverter
 from ftw.pdfgenerator.html2latex import wrapper
+from ftw.pdfgenerator.html2latex.utils import generate_manual_caption
 from ftw.pdfgenerator.utils import html2xmlentities
 from xml.dom import minidom
 import re
@@ -16,6 +17,26 @@ PLACEHOLDER_BOTTOM = interfaces.HTML2LATEX_CUSTOM_PATTERN_PLACEHOLDER_BOTTOM
 PREVENT_CHARACTER = interfaces.HTML2LATEX_PREVENT_CHARACTER
 LONGTABLE_ROWS_THRESHOLD = 15
 
+# Global border options:
+
+BORDER_TABLE_L = 2 ** 0
+BORDER_TABLE_R = 2 ** 1
+BORDER_TABLE_T = 2 ** 2
+BORDER_TABLE_B = 2 ** 3
+BORDER_TABLE = BORDER_TABLE_L | BORDER_TABLE_R | \
+    BORDER_TABLE_T | BORDER_TABLE_B
+
+BORDER_CELL_L = 2 ** 10
+BORDER_CELL_R = 2 ** 11
+BORDER_CELL_T = 2 ** 12
+BORDER_CELL_B = 2 ** 13
+BORDER_CELL = BORDER_CELL_L | BORDER_CELL_R | BORDER_CELL_T | BORDER_CELL_B
+
+NO_LAYOUT = 0
+GRID_LAYOUT = BORDER_TABLE | BORDER_CELL
+HORIZONTAL_LAYOUT = BORDER_TABLE_T | BORDER_TABLE_B | \
+    BORDER_CELL_T | BORDER_CELL_B
+
 
 class TableConverter(subconverter.SubConverter):
     """The TableConverter converts <table>-Tags to latex.
@@ -80,32 +101,47 @@ class TableConverter(subconverter.SubConverter):
             self._css_classes = classes.split(' ')
         return self._css_classes
 
-    def get_border(self):
+    def get_table_layout(self):
         domTable = self.dom.getElementsByTagName('table')[0]
 
         border = domTable.hasAttribute('border')
         if border and int(border) > 0:
-            return True
+            return GRID_LAYOUT
 
         elif 'border-grid' in self.get_css_classes() or \
-                'listing' in self.get_css_classes():
-            return True
+                'grid' in self.get_css_classes():
+            return GRID_LAYOUT
+
+        elif 'listing' in self.get_css_classes():
+            return HORIZONTAL_LAYOUT
 
         else:
-            return False
+            return NO_LAYOUT
 
     def render(self):
-        latex = '\\begin{%s}{%s}\n' % (
-            self.environment, self.get_table_format())
+        latex = [
+            r'\makeatletter\@ifundefined{tablewidth}{' + \
+                r'\newlength\tablewidth}\makeatother',
+            r'\setlength\tablewidth\linewidth',
+            r'\addtolength\tablewidth{-%i\tabcolsep}' % (
+                2 * len(self.columns))]
+
         caption_command, insert_caption_at_top = self.render_caption()
         if caption_command and insert_caption_at_top:
-            latex += caption_command
+            latex.append(caption_command.strip())
+            latex.append(r'\vspace{-\baselineskip}')
+
+        latex.append(r'\begin{%s}{%s}' % (
+                self.environment, self.get_table_format()))
+        latex.append(self.render_rows().strip())
+        latex.append(r'\end{%s}' % self.environment)
 
-        latex += self.render_rows()
         if caption_command and not insert_caption_at_top:
-            latex += caption_command
-        latex += '\\end{%s}\n' % self.environment
-        return latex
+            latex.append(r'\vspace{-\baselineskip}')
+            latex.append(caption_command.strip())
+
+        latex.append('')
+        return '\n'.join(latex)
 
     def render_rows(self):
         head_rows = []
@@ -139,14 +175,6 @@ class TableConverter(subconverter.SubConverter):
         returns the LaTeX-Command for the caption and weather the
         command should be inserted at the top or at the bottom of
         the table.
-
-        Example:
-        caption_command, insert_at_top = self.render_caption()
-        caption_command : \caption{My Table}
-        insert_at_top : True
-
-        If there is no command, it will return
-        None, False
         """
 
         caption = self.dom.getElementsByTagName('caption')
@@ -162,10 +190,13 @@ class TableConverter(subconverter.SubConverter):
             content = content.encode('utf8')
             latexContent = self.converter.convert(content)
 
-            return '\\caption%s{%s} \\\\\n' % (
-                ('notListed' in self.get_css_classes() and '*' or ''),
-                latexContent,
-                ), insert_at_top
+            show_in_index = 'notListed' not in self.get_css_classes()
+
+            latex = generate_manual_caption(latexContent, 'table',
+                                            show_in_index=show_in_index)
+
+            return latex, insert_at_top
+
         else:
             return None, False
 
@@ -352,18 +383,22 @@ class LatexColumn(object):
         return format_
 
     def has_left_border(self):
-        return self.table_converter.get_border()
+        return self.table_converter.get_table_layout() & BORDER_CELL_L
 
     def has_right_border(self):
-        if self.table_converter.get_border() and self.is_last_column():
-            return True
-
-        return False
+        if self.is_last_column():
+            return self.table_converter.get_table_layout() & BORDER_CELL_R
+        else:
+            return False
 
     def is_last_column(self):
         columns = self.table_converter.columns
         return columns[-1] == self
 
+    def is_first_column(self):
+        columns = self.table_converter.columns
+        return columns[0] == self
+
 
 class LatexRow(object):
     """
@@ -395,7 +430,7 @@ class LatexRow(object):
 
     def get_next_row(self):
         rows = self.table_converter.rows
-        next_row_index = rows.index(self)
+        next_row_index = rows.index(self) + 1
         try:
             return rows[next_row_index]
         except IndexError:
@@ -405,7 +440,7 @@ class LatexRow(object):
         latex = []
 
         if self.is_first_row():
-            line_latex = self.get_horizontal_line_latex([], self)
+            line_latex = self.get_horizontal_line_latex(self, None)
             if line_latex:
                 latex.append(line_latex)
 
@@ -418,30 +453,25 @@ class LatexRow(object):
                 latex_cells.append('')
 
         latex.append('%s \\\\' % ' & '.join(latex_cells))
-        line_latex = self.get_horizontal_line_latex(self, self.get_next_row())
+        line_latex = self.get_horizontal_line_latex(self.get_next_row(), self)
         if line_latex:
             latex.append(line_latex)
 
         return '\n'.join(latex) + '\n'
 
     def get_horizontal_line_latex(self, top_row, bottom_row):
-        lined_column_indexes = set()
+        lined_column_indexes = []
 
         if top_row:
-            index = 1
-            for cell in top_row.cells:
-                if cell.has_bottom_border_in_row(top_row):
-                    for i in range(index, index + cell.get_colspan()):
-                        lined_column_indexes.add(i)
-                index += cell.get_colspan()
+            lined_column_indexes.extend(
+                top_row._get_cell_indexes_with_border())
 
         if bottom_row:
-            index = 1
-            for cell in bottom_row.cells:
-                if cell.has_top_border_in_row(bottom_row):
-                    for i in range(index, index + cell.get_colspan()):
-                        lined_column_indexes.add(i)
-                index += cell.get_colspan()
+            lined_column_indexes.extend(
+                bottom_row._get_cell_indexes_with_border(bottom=True))
+
+        # make unique
+        lined_column_indexes = set(lined_column_indexes)
 
         if len(lined_column_indexes) == len(self.table_converter.columns):
             return r'\hline'
@@ -463,6 +493,27 @@ class LatexRow(object):
 
         return ''.join(latex)
 
+    def _get_cell_indexes_with_border(self, bottom=False):
+        """Decides for each cell of this row whether it should have a border
+        at the top (or at the bottom if `bottom` is `True`).
+        It returns each cells column index (starting with 1) for cells which
+        have a border.
+        """
+        if bottom:
+            has_border = lambda cell: cell.has_bottom_border_in_row(self)
+        else:
+            has_border = lambda cell: cell.has_top_border_in_row(self)
+
+        indexes = []
+        for cell in self.cells:
+            if not has_border(cell):
+                continue
+
+            for column in cell.columns:
+                indexes.append(self.table_converter.columns.index(column) + 1)
+
+        return indexes
+
 
 class LatexCell(object):
     """
@@ -506,8 +557,7 @@ class LatexCell(object):
             span = str(self.get_rowspan())
             latex = '\\multirow{%s}{%s}{%s}' % (
                 span,
-                str(self.get_calculated_width()).replace(r'\linewidth',
-                                                       r'\textwidth'),
+                str(self.get_calculated_width()),
                 self.render_content(),
                 )
         else:
@@ -561,7 +611,7 @@ class LatexCell(object):
         if 'scriptsize' in self.get_css_classes():
             latex = r'\scriptsize %s' % latex
 
-        if 'bold' in self.get_css_classes():
+        if 'bold' in self.get_css_classes() or self.is_head_cell():
             latex = r'\textbf{%s}' % latex
 
         if 'indent2' in self.get_css_classes():
@@ -575,9 +625,16 @@ class LatexCell(object):
         if 'thead' in [p.tagName.lower() for p in self.get_parent_nodes()]:
             # cell is within a <thead>
             return True
-        if self.dom_cell.tagName.lower() == 'th':
+
+        elif self.dom_cell.tagName.lower() == 'th':
             # cell is a <th>
             return True
+
+        elif 'vertical' in self.table_converter.get_css_classes() \
+                and self.columns[0].is_first_column():
+            # "vertical" class makes first column to be "head" cells.
+            return True
+
         else:
             # not a head cell
             return False
@@ -682,16 +739,16 @@ class LatexCell(object):
         """
         row_cells = self.rows[row_index].cells
         left_cell_index = row_cells.index(self) - 1
-        try:
-            return row_cells[left_cell_index]
-        except IndexError:
+        if left_cell_index < 0:
             return None
+        else:
+            return row_cells[left_cell_index]
 
     def has_bottom_border_in_row(self, row):
         if self.rows[-1] != row:
             return False
 
-        if self.table_converter.get_border():
+        if self.table_converter.get_table_layout() & BORDER_CELL_B:
             return True
 
         if 'border-bottom' in self.get_css_classes():
@@ -703,7 +760,7 @@ class LatexCell(object):
         if self.rows[0] != row:
             return False
 
-        if self.table_converter.get_border():
+        if self.table_converter.get_table_layout() & BORDER_CELL_T:
             return True
 
         if 'border-top' in self.get_css_classes():
@@ -743,12 +800,8 @@ class LatexCell(object):
 
     def get_css_classes(self):
         if self._css_classes is None:
-            if self.dom_cell:
-                classes = self.dom_cell.getAttribute('class').strip()
-                self._css_classes = classes.split(' ')
-
-            else:
-                self._css_classes = []
+            classes = self.dom_cell.getAttribute('class').strip()
+            self._css_classes = classes.split(' ')
 
         return self._css_classes
 
@@ -759,7 +812,7 @@ class LatexWidth(object):
     """
 
     TYPE_ABSOLUTE = 'absolute width (px, em, ..)'
-    TYPE_RELATIVE = 'relative width (% of linewidth, ..)'
+    TYPE_RELATIVE = 'relative width (% of tablewidth, ..)'
     VALID_ABSOLUTE_UNITS = [
         'cm',
         'mm',
@@ -812,7 +865,7 @@ class LatexWidth(object):
         if self.type == LatexWidth.TYPE_ABSOLUTE:
             return '%s%s' % (str(self.width), self.unit)
         elif self.type == LatexWidth.TYPE_RELATIVE:
-            return '%s\\linewidth' % str(self.width)
+            return '%s\\tablewidth' % str(self.width)
 
     def __str__(self):
         return self.get()
