@@ -1,11 +1,14 @@
 from ftw.pdfgenerator.html2latex.converter import HTML2LatexConverter
 from ftw.pdfgenerator.html2latex.subconverters import table
+from ftw.pdfgenerator.testing import ZCML_WITH_SITE_LAYER
 from ftw.testing import MockTestCase
 from unittest2 import TestCase
 
 
 class TestTableConverter(MockTestCase):
 
+    layer = ZCML_WITH_SITE_LAYER
+
     def convert(self, *args, **kwargs):
         count = 1
         if 'count' in kwargs:
@@ -51,8 +54,11 @@ class TestTableConverter(MockTestCase):
                 r'</table>'))
 
         latex = '\n'.join((
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-2\tabcolsep}',
                 r'\begin{tabular}{l}',
-                r'\multicolumn{1}{l}{My Head} \\',
+                r'\multicolumn{1}{l}{\textbf{My Head}} \\',
                 r'\multicolumn{1}{l}{My Body} \\',
                 r'\end{tabular}',
                 r''))
@@ -81,8 +87,12 @@ class TestTableConverter(MockTestCase):
                 r'</table>'))
 
         latex = '\n'.join((
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-4\tabcolsep}',
                 r'\begin{tabular}{ll}',
-                r'\multicolumn{1}{l}{headA} & \multicolumn{1}{l}{headB} \\',
+                r'\multicolumn{1}{l}{\textbf{headA}} & ' + \
+                    r'\multicolumn{1}{l}{\textbf{headB}} \\',
                 r'\multicolumn{1}{l}{1A} & \multicolumn{1}{l}{1B} \\',
                 r'\multicolumn{1}{l}{2A} & \multicolumn{1}{l}{2B} \\',
                 r'\end{tabular}',
@@ -109,13 +119,16 @@ class TestTableConverter(MockTestCase):
                 r'</table>'))
 
         latex = '\n'.join((
-                r'\begin{tabular}{p{0.3\linewidth}p{0.7\linewidth}}',
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-4\tabcolsep}',
+                r'\begin{tabular}{p{0.3\tablewidth}p{0.7\tablewidth}}',
 
-                r'\multicolumn{1}{p{0.3\linewidth}}{test1} & '
-                r'\multicolumn{1}{p{0.7\linewidth}}{test2} \\',
+                r'\multicolumn{1}{p{0.3\tablewidth}}{test1} & '
+                r'\multicolumn{1}{p{0.7\tablewidth}}{test2} \\',
 
-                r'\multicolumn{1}{p{0.3\linewidth}}{test3} & '
-                r'\multicolumn{1}{p{0.7\linewidth}}{test4} \\',
+                r'\multicolumn{1}{p{0.3\tablewidth}}{test3} & '
+                r'\multicolumn{1}{p{0.7\tablewidth}}{test4} \\',
 
                 r'\end{tabular}',
                 r''))
@@ -138,9 +151,12 @@ class TestTableConverter(MockTestCase):
                 r'</table>'))
 
         latex = '\n'.join((
-                r'\begin{tabular}{p{0.3\linewidth}r}',
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-4\tabcolsep}',
+                r'\begin{tabular}{p{0.3\tablewidth}r}',
 
-                r'\multicolumn{1}{p{0.3\linewidth}}{test1} & '
+                r'\multicolumn{1}{p{0.3\tablewidth}}{test1} & '
                 r'\multicolumn{1}{r}{test2} \\',
 
                 r'\end{tabular}',
@@ -170,14 +186,17 @@ class TestTableConverter(MockTestCase):
                 r'</table>'))
 
         latex = '\n'.join((
-                r'\begin{tabular}{p{0.3\linewidth}p{0.7\linewidth}}',
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-4\tabcolsep}',
+                r'\begin{tabular}{p{0.3\tablewidth}p{0.7\tablewidth}}',
 
-                r'\multicolumn{1}{p{0.3\linewidth}}{'
+                r'\multicolumn{1}{p{0.3\tablewidth}}{'
                 r'\raggedleft test1} & \multicolumn{1}{'
-                r'p{0.7\linewidth}}{\raggedright test2} \\',
+                r'p{0.7\tablewidth}}{\raggedright test2} \\',
 
-                r'\multicolumn{1}{p{0.3\linewidth}}{\center\vspace{-1.5em}'
-                r'test3} & \multicolumn{1}{p{0.7\linewidth}}{test4} \\',
+                r'\multicolumn{1}{p{0.3\tablewidth}}{\center\vspace{-1.5em}'
+                r'test3} & \multicolumn{1}{p{0.7\tablewidth}}{test4} \\',
 
                 r'\end{tabular}',
                 r''))
@@ -194,8 +213,23 @@ class TestTableConverter(MockTestCase):
                 r'</table>'))
 
         latex = '\n'.join((
+                r'\makeatletter\@ifundefined{tablewidth}{' + \
+                    r'\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-2\tabcolsep}',
+
+                r'\center{',
+                r'\addtocounter{table}{1}',
+                r'\addcontentsline{lot}{table}{' + \
+                    r'\protect\numberline ' + \
+                    r'{\thechapter.\arabic{table}}' + \
+                    r'{\ignorespaces Testtabelle}' + \
+                    r'}',
+                r'Table \thechapter.\arabic{table}: Testtabelle',
+                r'}',
+                r'\vspace{-\baselineskip}',
+
                 r'\begin{tabular}{l}',
-                r'\caption{Testtabelle} \\',
                 r'\multicolumn{1}{l}{test} \\',
                 r'\end{tabular}',
                 r''))
@@ -213,8 +247,17 @@ class TestTableConverter(MockTestCase):
                 r'</table>'))
 
         latex = '\n'.join((
+                r'\makeatletter\@ifundefined{tablewidth}{' + \
+                    r'\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-2\tabcolsep}',
+
+                r'\center{',
+                r'NotIndexedCaption',
+                r'}',
+                r'\vspace{-\baselineskip}',
+
                 r'\begin{tabular}{l}',
-                r'\caption*{NotIndexedCaption} \\',
                 r'\multicolumn{1}{l}{foo} \\',
                 r'\end{tabular}',
                 r''))
@@ -232,10 +275,26 @@ class TestTableConverter(MockTestCase):
                 r'</table>'))
 
         latex = '\n'.join((
+                r'\makeatletter\@ifundefined{tablewidth}{' + \
+                    r'\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-2\tabcolsep}',
+
                 r'\begin{tabular}{l}',
                 r'\multicolumn{1}{l}{foo} \\',
-                r'\caption{My Table} \\',
                 r'\end{tabular}',
+
+                r'\vspace{-\baselineskip}',
+                r'\center{',
+                r'\addtocounter{table}{1}',
+                r'\addcontentsline{lot}{table}{' + \
+                    r'\protect\numberline ' + \
+                    r'{\thechapter.\arabic{table}}' + \
+                    r'{\ignorespaces My Table}' + \
+                    r'}',
+                r'Table \thechapter.\arabic{table}: My Table',
+                r'}',
+
                 r''))
 
         self.assertEqual(self.convert(html), latex)
@@ -247,9 +306,17 @@ class TestTableConverter(MockTestCase):
             r'<table><tr><td>test2</td></tr></table>'
 
         latex = '\n'.join((
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength' + \
+                    r'\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-2\tabcolsep}',
                 r'\begin{tabular}{l}',
                 r'\multicolumn{1}{l}{test1} \\',
                 r'\end{tabular}',
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength' + \
+                    r'\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-2\tabcolsep}',
                 r'\begin{tabular}{l}',
                 r'\multicolumn{1}{l}{test2} \\',
                 r'\end{tabular}',
@@ -263,6 +330,9 @@ class TestTableConverter(MockTestCase):
         html = r'<table><tr><td><b>Hello</b> <i>World</i></td></tr></table>'
 
         latex = '\n'.join((
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-2\tabcolsep}',
                 r'\begin{tabular}{l}',
                 r'\multicolumn{1}{l}{{\bf Hello} {\it World}} \\',
                 r'\end{tabular}',
@@ -280,7 +350,10 @@ class TestTableConverter(MockTestCase):
 
         latex = '\n'.join((
                 r'This {\it is} a {\bf Table}:\\',
-                r' \begin{tabular}{l}',
+                r' \makeatletter\@ifundefined{tablewidth}{\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-2\tabcolsep}',
+                r'\begin{tabular}{l}',
                 r'\multicolumn{1}{l}{test} \\',
                 r'\end{tabular}',
                 r' {\bf yeah}'))
@@ -298,6 +371,9 @@ class TestTableConverter(MockTestCase):
                 r'</table>'))
 
         latex = '\n'.join((
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-2\tabcolsep}',
                 r'\begin{tabular}{l}',
                 r'\multicolumn{1}{l}{2>1} \\',
                 '\\multicolumn{1}{l}{X\xc3\xa4Y} \\\\',
@@ -327,15 +403,18 @@ class TestTableConverter(MockTestCase):
                 r'</table>'))
 
         latex = '\n'.join((
-                r'\begin{tabular}{p{0.33\linewidth}p{0.33\linewidth}'
-                r'p{0.34\linewidth}}',
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-6\tabcolsep}',
+                r'\begin{tabular}{p{0.33\tablewidth}p{0.33\tablewidth}'
+                r'p{0.34\tablewidth}}',
 
-                r'\multicolumn{1}{p{0.33\linewidth}}{one} & '
-                r'\multicolumn{1}{p{0.33\linewidth}}{two} & '
-                r'\multicolumn{1}{p{0.34\linewidth}}{three} \\',
+                r'\multicolumn{1}{p{0.33\tablewidth}}{one} & '
+                r'\multicolumn{1}{p{0.33\tablewidth}}{two} & '
+                r'\multicolumn{1}{p{0.34\tablewidth}}{three} \\',
 
-                r'\multicolumn{2}{p{0.66\linewidth}}{one and two} & '
-                r'\multicolumn{1}{p{0.34\linewidth}}{three} \\',
+                r'\multicolumn{2}{p{0.66\tablewidth}}{one and two} & '
+                r'\multicolumn{1}{p{0.34\tablewidth}}{three} \\',
 
                 r'\end{tabular}',
                 r''))
@@ -360,12 +439,15 @@ class TestTableConverter(MockTestCase):
                 r'</table>'))
 
         latex = '\n'.join((
-                r'\begin{tabular}{p{0.5\linewidth}p{0.5\linewidth}}',
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-4\tabcolsep}',
+                r'\begin{tabular}{p{0.5\tablewidth}p{0.5\tablewidth}}',
 
-                r'\multirow{2}{0.5\textwidth}{one} & \multicolumn{1}'
-                r'{p{0.5\linewidth}}{two} \\',
+                r'\multirow{2}{0.5\tablewidth}{one} & \multicolumn{1}'
+                r'{p{0.5\tablewidth}}{two} \\',
 
-                r' & \multicolumn{1}{p{0.5\linewidth}}{three} \\',
+                r' & \multicolumn{1}{p{0.5\tablewidth}}{three} \\',
                 r'\end{tabular}',
                 r''))
 
@@ -399,23 +481,27 @@ class TestTableConverter(MockTestCase):
                 ))
 
         latex = '\n'.join((
-                r'\begin{tabular}{p{0.25\linewidth}p{0.25\linewidth}' + \
-                    r'p{0.25\linewidth}p{0.25\linewidth}l}',
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength' + \
+                    r'\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-10\tabcolsep}',
+                r'\begin{tabular}{p{0.25\tablewidth}p{0.25\tablewidth}' + \
+                    r'p{0.25\tablewidth}p{0.25\tablewidth}l}',
 
                 # row 1
-                r'\multicolumn{1}{p{0.25\linewidth}}{A1} & ' + \
-                    r'\multirow{2}{0.25\textwidth}{B1-B2} & ' + \
-                    r'\multicolumn{1}{p{0.25\linewidth}}{C1} & ' + \
-                    r'\multicolumn{1}{p{0.25\linewidth}}{D1} \\',
+                r'\multicolumn{1}{p{0.25\tablewidth}}{A1} & ' + \
+                    r'\multirow{2}{0.25\tablewidth}{B1-B2} & ' + \
+                    r'\multicolumn{1}{p{0.25\tablewidth}}{C1} & ' + \
+                    r'\multicolumn{1}{p{0.25\tablewidth}}{D1} \\',
 
                 # row 2
-                r'\multicolumn{1}{p{0.25\linewidth}}{A2} & ' + \
+                r'\multicolumn{1}{p{0.25\tablewidth}}{A2} & ' + \
                     r' & '
-                    r'\multirow{2}{0.25\textwidth}{C2-C3} & ' + \
-                    r'\multicolumn{1}{p{0.25\linewidth}}{D2} \\',
+                    r'\multirow{2}{0.25\tablewidth}{C2-C3} & ' + \
+                    r'\multicolumn{1}{p{0.25\tablewidth}}{D2} \\',
 
                 # row 3
-                r'\multicolumn{2}{p{0.5\linewidth}}{A3-B3} & ' + \
+                r'\multicolumn{2}{p{0.5\tablewidth}}{A3-B3} & ' + \
                     r' & '
                     r'\multicolumn{2}{l}{D3-E3} \\',
 
@@ -444,16 +530,19 @@ class TestTableConverter(MockTestCase):
                 r'</table>'))
 
         latex = '\n'.join((
-                r'\begin{tabular}{|p{0.3\linewidth}|p{0.7\linewidth}|}',
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-4\tabcolsep}',
+                r'\begin{tabular}{|p{0.3\tablewidth}|p{0.7\tablewidth}|}',
                 r'\hline',
 
-                r'\multicolumn{1}{|p{0.3\linewidth}}{test1} & '
-                r'\multicolumn{1}{|p{0.7\linewidth}|}{test2} \\',
+                r'\multicolumn{1}{|p{0.3\tablewidth}}{test1} & '
+                r'\multicolumn{1}{|p{0.7\tablewidth}|}{test2} \\',
 
                 r'\hline',
 
-                r'\multicolumn{1}{|p{0.3\linewidth}}{test3} & '
-                r'\multicolumn{1}{|p{0.7\linewidth}|}{test4} \\',
+                r'\multicolumn{1}{|p{0.3\tablewidth}}{test3} & '
+                r'\multicolumn{1}{|p{0.7\tablewidth}|}{test4} \\',
 
                 r'\hline',
                 r'\end{tabular}',
@@ -476,11 +565,15 @@ class TestTableConverter(MockTestCase):
                 ))
 
         latex = '\n'.join((
-                r'\begin{tabular}{p{0.5\linewidth}p{0.5\linewidth}}',
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength' + \
+                    r'\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-4\tabcolsep}',
+                r'\begin{tabular}{p{0.5\tablewidth}p{0.5\tablewidth}}',
 
-                r'\multicolumn{1}{p{0.5\linewidth}}{' + \
+                r'\multicolumn{1}{p{0.5\tablewidth}}{' + \
                     r'hello\newline world} & ' + \
-                    r'\multicolumn{1}{p{0.5\linewidth}}{' + \
+                    r'\multicolumn{1}{p{0.5\tablewidth}}{' + \
                     r'foo\newline \newline bar} \\',
 
                 r'\end{tabular}',
@@ -504,11 +597,14 @@ class TestTableConverter(MockTestCase):
                 ))
 
         tabular_latex = '\n'.join((
-                r'\begin{tabular}{p{0.5\linewidth}p{0.5\linewidth}}',
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-4\tabcolsep}',
+                r'\begin{tabular}{p{0.5\tablewidth}p{0.5\tablewidth}}',
 
-                r'\multicolumn{1}{p{0.5\linewidth}}{' + \
+                r'\multicolumn{1}{p{0.5\tablewidth}}{' + \
                     r'foo} & ' + \
-                    r'\multicolumn{1}{p{0.5\linewidth}}{' + \
+                    r'\multicolumn{1}{p{0.5\tablewidth}}{' + \
                     r'bar} \\',
 
                 r'\end{tabular}',
@@ -529,6 +625,9 @@ class TestTableConverter(MockTestCase):
                 ))
 
         longtable_latex = '\n'.join((
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-2\tabcolsep}',
                 r'\begin{longtable}{l}',
                 (40 * '\\multicolumn{1}{l}{foo} \\\\\n').strip(),
                 r'\end{longtable}',
@@ -550,11 +649,14 @@ class TestTableConverter(MockTestCase):
                 ))
 
         longtable_latex = '\n'.join((
-                r'\begin{longtable}{p{0.5\linewidth}p{0.5\linewidth}}',
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-4\tabcolsep}',
+                r'\begin{longtable}{p{0.5\tablewidth}p{0.5\tablewidth}}',
 
-                r'\multicolumn{1}{p{0.5\linewidth}}{' + \
+                r'\multicolumn{1}{p{0.5\tablewidth}}{' + \
                     r'foo} & ' + \
-                    r'\multicolumn{1}{p{0.5\linewidth}}{' + \
+                    r'\multicolumn{1}{p{0.5\tablewidth}}{' + \
                     r'bar} \\',
 
                 r'\end{longtable}',
@@ -573,6 +675,9 @@ class TestTableConverter(MockTestCase):
                 ))
 
         tabular_latex = '\n'.join((
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-2\tabcolsep}',
                 r'\begin{tabular}{l}',
                 (20 * '\\multicolumn{1}{l}{foo} \\\\\n').strip(),
                 r'\end{tabular}',
@@ -645,11 +750,14 @@ class TestTableConverter(MockTestCase):
                 r'</table>'))
 
         latex = '\n'.join((
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-4\tabcolsep}',
                 r'\begin{tabular}{|l|l|}',
                 r'\hline',
 
-                r'\multicolumn{1}{|l}{heading A} & ' + \
-                    r'\multicolumn{1}{|l|}{heading B} \\',
+                r'\multicolumn{1}{|l}{\textbf{heading A}} & ' + \
+                    r'\multicolumn{1}{|l|}{\textbf{heading B}} \\',
                 r'\hline',
 
                 r'\multicolumn{1}{|l}{content 1A} & ' + \
@@ -667,6 +775,106 @@ class TestTableConverter(MockTestCase):
         self.assertEqual(self.convert(html), latex)
 
     def test_complex_grid_css_class(self):
+        html = '\n'.join((
+                r'<table class="no-page-break grid">',
+                r' <colgroup>',
+                r'  <col width="50%" />'
+                r'  <col width="25%" />'
+                r'  <col width="25%" />'
+                r' </colgroup>',
+                r' <thead>',
+                r'  <tr>',
+                r'   <th colspan="3">heading</th>',
+                r'  </tr>',
+                r' </thead>',
+                r' <tbody>',
+                r'  <tr>',
+                r'   <td>content 1A</td>',
+                r'   <td rowspan="2">content 1/2 B</td>',
+                r'   <td>content 1C</td>',
+                r'  </tr>',
+                r'  <tr>',
+                r'   <td>content 2A</td>',
+                r'   <td>content 2C</td>',
+                r'  </tr>',
+                r' </tbody>',
+                r'</table>'))
+
+        latex = '\n'.join((
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-6\tabcolsep}',
+                r'\begin{tabular}{|p{0.5\tablewidth}|p{0.25\tablewidth}|' + \
+                    r'p{0.25\tablewidth}|}',
+                r'\hline',
+
+                r'\multicolumn{3}{|p{1.0\tablewidth}|}{\textbf{heading}} \\',
+                r'\hline',
+
+                r'\multicolumn{1}{|p{0.5\tablewidth}|}{content 1A} & ' + \
+                    r'\multirow{2}{0.25\tablewidth}{content 1/2 B} & ' + \
+                    r'\multicolumn{1}{|p{0.25\tablewidth}|}{content 1C} \\',
+                r'\cline{1-1}\cline{3-3}',
+
+                r'\multicolumn{1}{|p{0.5\tablewidth}|}{content 2A} & ' + \
+                    r' & '
+                    r'\multicolumn{1}{|p{0.25\tablewidth}|}{content 2C} \\',
+                r'\hline',
+
+                r'\end{tabular}',
+                r''
+                ))
+
+        self.assertEqual(self.convert(html), latex)
+
+    def test_border_rowspan_collapse(self):
+        html = '\n'.join((
+                r'<table class="no-page-break">',
+                r' <colgroup>',
+                r'  <col width="25%" />'
+                r'  <col width="25%" />'
+                r'  <col width="25%" />'
+                r'  <col width="25%" />'
+                r' </colgroup>',
+                r' <tbody>',
+                r'  <tr>',
+                r'   <td rowspan="2">1A</td>',
+                r'   <td>content 1B</td>',
+                r'   <td rowspan="2" class="border-right border-left">' + \
+                    r'content 1/2 C</td>',
+                r'   <td>content 1D</td>',
+                r'  </tr>',
+                r'  <tr>',
+                r'   <td>content 2B</td>',
+                r'   <td>content 2D</td>',
+                r'  </tr>',
+                r' </tbody>',
+                r'</table>'))
+
+        latex = '\n'.join((
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-8\tabcolsep}',
+                r'\begin{tabular}{p{0.25\tablewidth}p{0.25\tablewidth}' + \
+                    r'p{0.25\tablewidth}p{0.25\tablewidth}}',
+
+                r'\multirow{2}{0.25\tablewidth}{1A} & ' + \
+                r'\multicolumn{1}{p{0.25\tablewidth}|}{content 1B} & ' + \
+                    r'\multirow{2}{0.25\tablewidth}{content 1/2 C} & ' + \
+                    r'\multicolumn{1}{|p{0.25\tablewidth}}{content 1D} \\',
+
+                r' & ' + \
+                    r'\multicolumn{1}{p{0.25\tablewidth}|}{content 2B} & ' + \
+                    r' & '
+                    r'\multicolumn{1}{|p{0.25\tablewidth}}{content 2D} \\',
+
+                r'\end{tabular}',
+                r''
+                ))
+
+        self.assertEqual(self.convert(html), latex)
+
+    def test_listing_css_class(self):
         html = '\n'.join((
                 r'<table class="no-page-break listing">',
                 r' <colgroup>',
@@ -693,21 +901,24 @@ class TestTableConverter(MockTestCase):
                 r'</table>'))
 
         latex = '\n'.join((
-                r'\begin{tabular}{|p{0.5\linewidth}|p{0.25\linewidth}|' + \
-                    'p{0.25\linewidth}|}',
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-6\tabcolsep}',
+                r'\begin{tabular}{p{0.5\tablewidth}p{0.25\tablewidth}' + \
+                    r'p{0.25\tablewidth}}',
                 r'\hline',
 
-                r'\multicolumn{3}{|p{1.0\linewidth}|}{heading} \\',
+                r'\multicolumn{3}{p{1.0\tablewidth}}{\textbf{heading}} \\',
                 r'\hline',
 
-                r'\multicolumn{1}{|p{0.5\linewidth}|}{content 1A} & ' + \
-                    r'\multirow{2}{0.25\textwidth}{content 1/2 B} & ' + \
-                    r'\multicolumn{1}{|p{0.25\linewidth}|}{content 1C} \\',
-                r'\hline',
+                r'\multicolumn{1}{p{0.5\tablewidth}}{content 1A} & ' + \
+                    r'\multirow{2}{0.25\tablewidth}{content 1/2 B} & ' + \
+                    r'\multicolumn{1}{p{0.25\tablewidth}}{content 1C} \\',
+                r'\cline{1-1}\cline{3-3}',
 
-                r'\multicolumn{1}{|p{0.5\linewidth}|}{content 2A} & ' + \
+                r'\multicolumn{1}{p{0.5\tablewidth}}{content 2A} & ' + \
                     r' & '
-                    r'\multicolumn{1}{|p{0.25\linewidth}|}{content 2C} \\',
+                    r'\multicolumn{1}{p{0.25\tablewidth}}{content 2C} \\',
                 r'\hline',
 
                 r'\end{tabular}',
@@ -716,18 +927,67 @@ class TestTableConverter(MockTestCase):
 
         self.assertEqual(self.convert(html), latex)
 
+    def test_vertical_css_class_makes_first_column_bold(self):
+        html = '\n'.join((
+                r'<table class="no-page-break vertical">',
+                r' <colgroup>',
+                r'  <col width="50%" />'
+                r'  <col width="25%" />'
+                r'  <col width="25%" />'
+                r' </colgroup>',
+                r' <thead>',
+                r'  <tr>',
+                r'   <th colspan="3">heading</th>',
+                r'  </tr>',
+                r' </thead>',
+                r' <tbody>',
+                r'  <tr>',
+                r'   <td>content 1A</td>',
+                r'   <td rowspan="2">content 1/2 B</td>',
+                r'   <td>content 1C</td>',
+                r'  </tr>',
+                r'  <tr>',
+                r'   <td>content 2A</td>',
+                r'   <td>content 2C</td>',
+                r'  </tr>',
+                r' </tbody>',
+                r'</table>'))
+
+        latex = '\n'.join((
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-6\tabcolsep}',
+                r'\begin{tabular}{p{0.5\tablewidth}p{0.25\tablewidth}' + \
+                    r'p{0.25\tablewidth}}',
+
+                r'\multicolumn{3}{p{1.0\tablewidth}}{\textbf{heading}} \\',
+
+                r'\multicolumn{1}{p{0.5\tablewidth}}' + \
+                    r'{\textbf{content 1A}} & ' + \
+                    r'\multirow{2}{0.25\tablewidth}{content 1/2 B} & ' + \
+                    r'\multicolumn{1}{p{0.25\tablewidth}}{content 1C} \\',
+
+                r'\multicolumn{1}{p{0.5\tablewidth}}' + \
+                    r'{\textbf{content 2A}} & ' + \
+                    r' & ' + \
+                    r'\multicolumn{1}{p{0.25\tablewidth}}{content 2C} \\',
+
+                r'\end{tabular}',
+                r''
+                ))
+
+        self.assertEqual(self.convert(html), latex)
+
     def test_border_classes(self):
         html = '\n'.join((
                 r'<table class="no-page-break">',
-                r' <thead>',
-                r'  <tr>',
-                r'   <th class="border-right">A1</th>',
-                r'   <th class="border-top">B1</th>',
-                r'   <th class="border-left border-bottom">C1</th>',
-                r'  </tr>',
-                r' </thead>',
                 r' <tbody>',
                 r'  <tr>',
+                r'   <td class="border-right">A1</td>',
+                r'   <td class="border-top">B1</td>',
+                r'   <td class="border-left border-bottom">C1</td>',
+                r'  </tr>',
+                r'  <tr>',
                 r'   <td class="border-right">A2</td>',
                 r'   <td class="border-top border-bottom">B2</td>',
                 r'   <td class="border-left border-bottom">C2</td>',
@@ -738,6 +998,9 @@ class TestTableConverter(MockTestCase):
 
 
         latex = '\n'.join((
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-6\tabcolsep}',
                 r'\begin{tabular}{lll}',
 
                 r'\cline{2-2}',
@@ -780,6 +1043,9 @@ class TestTableConverter(MockTestCase):
 
         latex = '\n'.join((
                 r'without width: '
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-2\tabcolsep}',
                 r'\begin{tabular}{l}',
                 r'\multicolumn{1}{l}{left} \\',
                 r'\multicolumn{1}{c}{center} \\',
@@ -787,11 +1053,14 @@ class TestTableConverter(MockTestCase):
                 r'\end{tabular}',
 
                 r' with width: '
-                r'\begin{tabular}{p{1.0\linewidth}}',
-                r'\multicolumn{1}{p{1.0\linewidth}}{\raggedright left} \\',
-                r'\multicolumn{1}{p{1.0\linewidth}}{' + \
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-2\tabcolsep}',
+                r'\begin{tabular}{p{1.0\tablewidth}}',
+                r'\multicolumn{1}{p{1.0\tablewidth}}{\raggedright left} \\',
+                r'\multicolumn{1}{p{1.0\tablewidth}}{' + \
                     r'\center\vspace{-1.5em}center} \\',
-                r'\multicolumn{1}{p{1.0\linewidth}}{\raggedleft right} \\',
+                r'\multicolumn{1}{p{1.0\tablewidth}}{\raggedleft right} \\',
                 r'\end{tabular}',
                 r''
                 ))
@@ -809,11 +1078,14 @@ class TestTableConverter(MockTestCase):
                 ))
 
         latex = '\n'.join((
-                r'\begin{tabular}{p{0.5\linewidth}p{0.5\linewidth}}',
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-4\tabcolsep}',
+                r'\begin{tabular}{p{0.5\tablewidth}p{0.5\tablewidth}}',
 
-                r'\multicolumn{1}{p{0.5\linewidth}}{' + \
+                r'\multicolumn{1}{p{0.5\tablewidth}}{' + \
                     r'\hangindent 0.2cm\hspace{0.2cm} foo} & ' + \
-                    r'\multicolumn{1}{p{0.5\linewidth}}{' + \
+                    r'\multicolumn{1}{p{0.5\tablewidth}}{' + \
                     r'bar} \\',
 
                 r'\end{tabular}',
@@ -833,11 +1105,14 @@ class TestTableConverter(MockTestCase):
                 ))
 
         latex = '\n'.join((
-                r'\begin{tabular}{p{0.5\linewidth}p{0.5\linewidth}}',
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-4\tabcolsep}',
+                r'\begin{tabular}{p{0.5\tablewidth}p{0.5\tablewidth}}',
 
-                r'\multicolumn{1}{p{0.5\linewidth}}{' + \
+                r'\multicolumn{1}{p{0.5\tablewidth}}{' + \
                     r'\hangindent 1cm\hspace{1cm} foo} & ' + \
-                    r'\multicolumn{1}{p{0.5\linewidth}}{' + \
+                    r'\multicolumn{1}{p{0.5\tablewidth}}{' + \
                     r'bar} \\',
 
                 r'\end{tabular}',
@@ -857,11 +1132,14 @@ class TestTableConverter(MockTestCase):
                 ))
 
         latex = '\n'.join((
-                r'\begin{tabular}{p{0.5\linewidth}p{0.5\linewidth}}',
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-4\tabcolsep}',
+                r'\begin{tabular}{p{0.5\tablewidth}p{0.5\tablewidth}}',
 
-                r'\multicolumn{1}{p{0.5\linewidth}}{' + \
+                r'\multicolumn{1}{p{0.5\tablewidth}}{' + \
                     r'\textbf{foo}} & ' + \
-                    r'\multicolumn{1}{p{0.5\linewidth}}{' + \
+                    r'\multicolumn{1}{p{0.5\tablewidth}}{' + \
                     r'\hangindent 1cm\hspace{1cm} \textbf{bar}} \\',
 
                 r'\end{tabular}',
@@ -881,11 +1159,14 @@ class TestTableConverter(MockTestCase):
                 ))
 
         latex = '\n'.join((
-                r'\begin{tabular}{p{0.5\linewidth}p{0.5\linewidth}}',
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-4\tabcolsep}',
+                r'\begin{tabular}{p{0.5\tablewidth}p{0.5\tablewidth}}',
 
-                r'\multicolumn{1}{p{0.5\linewidth}}{' + \
+                r'\multicolumn{1}{p{0.5\tablewidth}}{' + \
                     r'\textcolor{gray}{foo}} & ' + \
-                    r'\multicolumn{1}{p{0.5\linewidth}}{' + \
+                    r'\multicolumn{1}{p{0.5\tablewidth}}{' + \
                     r'bar} \\',
 
                 r'\end{tabular}',
@@ -906,11 +1187,14 @@ class TestTableConverter(MockTestCase):
                 ))
 
         latex = '\n'.join((
-                r'\begin{tabular}{p{0.5\linewidth}p{0.5\linewidth}}',
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-4\tabcolsep}',
+                r'\begin{tabular}{p{0.5\tablewidth}p{0.5\tablewidth}}',
 
-                r'\multicolumn{1}{p{0.5\linewidth}}{' + \
+                r'\multicolumn{1}{p{0.5\tablewidth}}{' + \
                     r'\footnotesize foo} & ' + \
-                    r'\multicolumn{1}{p{0.5\linewidth}}{' + \
+                    r'\multicolumn{1}{p{0.5\tablewidth}}{' + \
                     r'bar} \\',
 
                 r'\end{tabular}',
@@ -930,11 +1214,14 @@ class TestTableConverter(MockTestCase):
                 ))
 
         latex = '\n'.join((
-                r'\begin{tabular}{p{0.5\linewidth}p{0.5\linewidth}}',
+                r'\makeatletter\@ifundefined{tablewidth}{\newlength\tablewidth}\makeatother',
+                r'\setlength\tablewidth\linewidth',
+                r'\addtolength\tablewidth{-4\tabcolsep}',
+                r'\begin{tabular}{p{0.5\tablewidth}p{0.5\tablewidth}}',
 
-                r'\multicolumn{1}{p{0.5\linewidth}}{' + \
+                r'\multicolumn{1}{p{0.5\tablewidth}}{' + \
                     r'\scriptsize foo} & ' + \
-                    r'\multicolumn{1}{p{0.5\linewidth}}{' + \
+                    r'\multicolumn{1}{p{0.5\tablewidth}}{' + \
                     r'bar} \\',
 
                 r'\end{tabular}',
@@ -967,13 +1254,13 @@ class TestLatexWidth(TestCase):
 
     def test_relative_width(self):
         width1 = table.LatexWidth.convert('10%')
-        self.assertEqual(str(width1), r'0.1\linewidth')
+        self.assertEqual(str(width1), r'0.1\tablewidth')
 
         width2 = table.LatexWidth.convert('25%')
-        self.assertEqual(str(width2), r'0.25\linewidth')
+        self.assertEqual(str(width2), r'0.25\tablewidth')
 
         self.assertEqual(str(width1 + width2),
-                         r'0.35\linewidth')
+                         r'0.35\tablewidth')
 
     def test_defaults_to_em(self):
         width1 = table.LatexWidth.convert('50')
